@@ -154,7 +154,7 @@ int	size;
 		if (!failure)
 			return size;
 	    }
-	/* return -1;	/* Never reached */
+	/* return -1; */	/* Never reached */
 }
 
 /*
@@ -358,7 +358,6 @@ aClient	*cptr;
 {
 	Reg	aClient	*tmp, *prev = NULL;
 	Reg	int	hashv;
-	int	hv;
 
 	hashv = cptr->hashv;
 	if (hashv < 0)
@@ -405,7 +404,6 @@ aChannel	*chptr;
 {
 	Reg	aChannel	*tmp, *prev = NULL;
 	Reg	int	hashv;
-	int	hv;
 
 	hashv = chptr->hashv;
 	if (hashv < 0)
@@ -448,7 +446,6 @@ aClient	*cptr;
 {
 	Reg	aServer	*tmp, *prev = NULL;
 	Reg	int	hashv;
-	int	hv;
 
 	hashv = sptr->stok * 15053;
 	if (hashv < 0)
@@ -776,7 +773,7 @@ char	*parv[];
 		for (i = 0; i < 10; i++)
 			sendto_one(sptr,"NOTICE %s :Entires with %d links : %d",
 			parv[0], i, link_pop[i]);
-		return (0);
+		return (1);
 	case 'r' :
 	    {
 		Reg	aClient	*acptr;
@@ -802,13 +799,13 @@ char	*parv[];
 			sendto_one(sptr,"NOTICE %s :%s hash to entry %d",
 				   parv[0], parv[2],
 				   hash_channel_name(parv[2]));
-		return (0);
+		return (1);
 	case 'h' :
 		if (parc > 2)
 			sendto_one(sptr,"NOTICE %s :%s hash to entry %d",
 				   parv[0], parv[2],
 				   hash_nick_name(parv[2]));
-		return (0);
+		return (1);
 	case 'n' :
 	    {
 		aClient	*tmp;
@@ -830,7 +827,7 @@ char	*parv[];
 				sendto_one(sptr,"NOTICE %s :Node: %d #%d %s",
 					   parv[0], l, i, tmp->name);
 			    }
-		return (0);
+		return (1);
 	    }
 	case 'N' :
 	    {
@@ -849,7 +846,7 @@ char	*parv[];
 			     i++, tmp = tmp->hnextch)
 				sendto_one(sptr,"NOTICE %s :Node: %d #%d %s",
 					   parv[0], l, i, tmp->chname);
-		return (0);
+		return (1);
 	    }
 	case 'S' :
 #else
@@ -859,16 +856,14 @@ char	*parv[];
 	sendto_one(sptr, "NOTICE %s :[SSERV] [IRCDC]", parv[0]);
 	sendto_one(sptr, "NOTICE %s :[CHANC] [SMISC]", parv[0]);
 	sendto_one(sptr, "NOTICE %s :[HASHC] [VERSH]", parv[0]);
-	sendto_one(sptr, "NOTICE %s :[SBSDC] HOSTID", parv[0]);
+	sendto_one(sptr, "NOTICE %s :[MAKEF] HOSTID", parv[0]);
 #ifndef	DEBUGMODE
 	}
 #endif
-	    return 0;
+	    return 1;
 #ifdef	DEBUGMODE
 	case 'z' :
 	    {
-		Reg	aClient	*acptr;
-
 		if (parc <= 2)
 			return 0;
 		l = atoi(parv[2]);
@@ -880,15 +875,12 @@ char	*parv[];
 	    }
 	case 'Z' :
 	    {
-		Reg	aChannel	*acptr;
-
 		if (parc <= 2)
 			return 0;
 		l = atoi(parv[2]);
 		if (l < 256)
 			return 0;
-		bigger_hash_table(&_CHANNELHASHSIZE,
-						 channelTable, l);
+		bigger_hash_table(&_CHANNELHASHSIZE, channelTable, l);
 		sendto_one(sptr, "NOTICE %s :CHANNELHASHSIZE now %d",
 			   parv[0], l);
 		break;
@@ -916,6 +908,6 @@ char	*parv[];
 		   parv[0], clhits, clmiss);
 	sendto_one(sptr,"NOTICE %s :Channel hits %d miss %d",
 		   parv[0], chhits, chmiss);
-	return 0;
+	return 1;
 #endif
 }
