@@ -350,32 +350,41 @@ char	*nick, *username;
 	    {
 		if ((i = check_client(sptr)))
 		    {
-			sendto_flag(SCH_LOCAL, "%s from %s.", (i == -4) ?
+			ircstp->is_ref++;
+			sptr->exitc = EXITC_REF;
+			sendto_flag(SCH_LOCAL, "%s from %s.", (i == -5) ?
+				    "Too many user connections" : (i == -4) ?
 				    "Too many host connections" :
 				    (i == -3) ? "Too many connections" :
 			 	    "Unauthorized connection",
 				    get_client_host(sptr));
-			ircstp->is_ref++;
 #if defined(USE_SYSLOG) && defined(SYSLOG_CONN)
 			syslog(LOG_NOTICE, "%s ( %s ): <none>@%s [%s] %c\n",
+			       myctime(sptr->firsttime),
+			       (i == -5) ? " u@h max " :
 			       (i == -4) ? " IP  max " : (i == -3) ? 
 			       " No more " : " No Auth ",
 			       (IsUnixSocket(sptr)) ? me.sockhost :
 			       ((sptr->hostp) ? sptr->hostp->h_name :
-				sptr->sockhost), sptr->username, '-');
+				sptr->sockhost), sptr->username, sptr->exitc);
 #endif		    
 #ifdef FNAME_CONNLOG
 			sendto_flog(myctime(sptr->firsttime), 
+				    (i == -5) ? " u@h max " :
 				    (i == -4) ? " IP  max " : (i == -3) ? 
 				    " No more " : " No Auth ", 0, "<none>",
 				    (IsUnixSocket(sptr)) ? me.sockhost :
 				    ((sptr->hostp) ? sptr->hostp->h_name :
-				    sptr->sockhost), sptr->username, '-');
+				    sptr->sockhost), sptr->username,
+				    &sptr->exitc);
 #endif
-			return exit_client(cptr, sptr, &me, (i == -4) ?
-				   "No more connections from your host" :
-					   (i == -3) ? "No more connections" :
-					   "No Authorization");
+			return exit_client(cptr, sptr, &me,
+				(i == -5) ?
+					"No more connections from you" :
+				(i == -4) ?
+					"No more connections from your host" :
+				(i == -3) ? "No more connections" :
+					    "No Authorization");
 		    }
 		aconf = sptr->confs->value.aconf;
 		if (IsUnixSocket(sptr))
@@ -443,6 +452,7 @@ char	*nick, *username;
 			sendto_flag(SCH_LOCAL, "K-lined %s@%s.",
 				    sptr->user->username, sptr->sockhost);
 			ircstp->is_ref++;
+			sptr->exitc = EXITC_REF;
 #if defined(USE_SYSLOG) && defined(SYSLOG_CONN)
 			syslog(LOG_NOTICE, "%s ( K lined ): %s@%s [%s] %c\n",
 			       myctime(sptr->firsttime), sptr->user->username,
@@ -451,7 +461,7 @@ char	*nick, *username;
 #ifdef FNAME_CONNLOG
 			sendto_flog(myctime(sptr->firsttime), " K lined ", 0,
 				    sptr->user->username, sptr->user->host,
-				    sptr->username, '-');
+				    sptr->username, "-");
 #endif
 			return exit_client(cptr, sptr, &me, "K-lined");
 		    }
@@ -461,6 +471,7 @@ char	*nick, *username;
 			sendto_flag(SCH_LOCAL, "R-lined %s@%s.",
 				    sptr->user->username, sptr->sockhost);
 			ircstp->is_ref++;
+			sptr->exitc = EXITC_REF;
 # if defined(USE_SYSLOG) && defined(SYSLOG_CONN)
 			syslog(LOG_NOTICE, "%s ( R lined ): %s@%s [%s] %c\n",
 			       myctime(sptr->firsttime), sptr->user->username,
@@ -469,7 +480,7 @@ char	*nick, *username;
 # ifdef FNAME_CONNLOG
 			sendto_flog(myctime(sptr->firsttime), " R lined ", 0,
 				    sptr->user->username, sptr->user->host,
-				    sptr->username, '-');
+				    sptr->username, "-");
 # endif
 			return exit_client(cptr, sptr, &me , "R-lined");
 		    }
@@ -604,13 +615,13 @@ char	*parv[];
 	if (IsService(sptr))
    	    {
 		sendto_one(sptr, err_str(ERR_ALREADYREGISTRED, parv[0]));
-		return 0;
+		return 1;
 	    }
 
 	if (parc < 2)
 	    {
 		sendto_one(sptr, err_str(ERR_NONICKNAMEGIVEN, parv[0]));
-		return 0;
+		return 1;
 	    }
 	if (MyConnect(sptr) && (s = (char *)index(parv[1], '~')))
 		*s = '\0';
@@ -663,7 +674,7 @@ char	*parv[];
 				return exit_client(cptr,sptr,&me,"BadNick");
 			    }
 		    }
-		return 0;
+		return 1;
 	    }
 
 	/*
@@ -677,9 +688,9 @@ char	*parv[];
 	if ((acptr = find_server(nick, NULL)))
 		if (MyConnect(sptr))
 		    {
-			sendto_one(sptr, err_str(ERR_NICKNAMEINUSE,
-				   BadPtr(parv[0]) ? "*" : parv[0]), nick);
-			return 0; /* NICK message ignored */
+			sendto_one(sptr, err_str(ERR_NICKNAMEINUSE, parv[0]),
+				   nick);
+			return 1; /* NICK message ignored */
 		    }
 	/*
 	** acptr already has result from previous find_server()
@@ -736,7 +747,7 @@ char	*parv[];
 			** especially since servers prior to this
 			** version would treat it as nick collision.
 			*/
-			return 0; /* NICK Message ignored */
+			return 1; /* NICK Message ignored */
 	/*
 	** Note: From this point forward it can be assumed that
 	** acptr != sptr (point to different client structures).
@@ -762,10 +773,9 @@ char	*parv[];
 		** send error reply and ignore the command.
 		*/
 		sendto_one(sptr, err_str((delayed) ? ERR_UNAVAILRESOURCE
-					 : ERR_NICKNAMEINUSE,
-			   /* parv[0] is empty when connecting */
-			   BadPtr(parv[0]) ? "*" : parv[0]), nick);
-		return 0; /* NICK message ignored */
+						   : ERR_NICKNAMEINUSE,
+					 parv[0]), nick);
+		return 1; /* NICK message ignored */
 	    }
 	/*
 	** NICK was coming from a server connection. Means that the same
@@ -869,12 +879,12 @@ nickkilldone:
 		if (MyConnect(sptr))
 		{
 			if (!IsPerson(sptr))    /* Unregistered client */
-				return 0;       /* Ignore new NICKs */
+				return 1;       /* Ignore new NICKs */
 			if (IsRestricted(sptr))
 			    {
 				sendto_one(sptr,
 					   err_str(ERR_RESTRICTED, nick));
-				return 0;
+				return 1;
 			    }
 		}
 		/*
@@ -921,7 +931,7 @@ nickkilldone:
 	**  Finally set new nick name.
 	*/
 	(void)add_to_client_hash_table(nick, sptr);
-	return 0;
+	return 2;
 }
 
 /*
@@ -946,27 +956,41 @@ int	notice;
 	Reg	char	*s;
 	aChannel *chptr;
 	char	*nick, *server, *p, *cmd, *host;
-	int	count = 0;
+	int	count = 0, nickcnt = 0;
 
 	cmd = notice ? MSG_NOTICE : MSG_PRIVATE;
 
 	if (parc < 2 || *parv[1] == '\0')
 	    {
 		sendto_one(sptr, err_str(ERR_NORECIPIENT, parv[0]), cmd);
-		return 0;
+		return 1;
 	    }
 
 	if (parc < 3 || *parv[2] == '\0')
 	    {
 		sendto_one(sptr, err_str(ERR_NOTEXTTOSEND, parv[0]));
-		return 0;
+		return 1;
 	    }
 
 	if (MyConnect(sptr))
 		parv[1] = canonize(parv[1]);
 	for (p = NULL, nick = strtoken(&p, parv[1], ","); nick;
-	     nick = strtoken(&p, NULL, ","))
+	     nick = strtoken(&p, NULL, ","), nickcnt++)
 	    {
+
+		/*
+		** restrict destination list to MAXMSGDESTS recipients to
+		** solve SPAM problem --Yegg
+		*/
+#define		MAXMSGDESTS 5
+		if (nickcnt >= MAXMSGDESTS) {
+		    if (!notice)
+			sendto_one(sptr, err_str(
+				   ERR_TOOMANYDESTS,
+				   parv[0]), nick);
+		    continue;
+		}
+
 		/*
 		** nickname addressed?
 		*/
@@ -1102,7 +1126,7 @@ int	notice;
 		    }
 		sendto_one(sptr, err_str(ERR_NOSUCHNICK, parv[0]), nick);
 	    }
-    return 0;
+    return nickcnt;
 }
 
 /*
@@ -1178,14 +1202,14 @@ char	*parv[];
 	aChannel *mychannel;
 	char	*channame = NULL, *s;
 	int	oper = parc > 2 ? (*parv[2] == 'o' ): 0; /* Show OPERS only */
-	int	member;
+	int	member, penalty = 0;
 
 	if (!BadPtr(mask))
 	    {
 		if ((s = (char *)index(mask, ',')))
 		    {
 			parv[1] = ++s;
-			(void)m_who(cptr, sptr, parc, parv);
+			penalty = m_who(cptr, sptr, parc, parv);
 		    }
 		clean_channelname(mask);
 	    }
@@ -1195,8 +1219,6 @@ char	*parv[];
 		if ((lp = sptr->user->channel))
 			mychannel = lp->value.chptr;
 
-	/* Allow use of m_who without registering */
-	
 	/*
 	**  Following code is some ugly hacking to preserve the
 	**  functions of the old implementation. (Also, people
@@ -1224,78 +1246,89 @@ char	*parv[];
 		 */
 		chptr = find_channel(channame, NULL);
 		if (chptr && !IsAnonymous(chptr))
-		  {
-		    member = IsMember(sptr, chptr);
-		    if (member || !SecretChannel(chptr))
-			for (lp = chptr->members; lp; lp = lp->next)
-			    {
-				if (oper && !IsAnOper(lp->value.cptr))
-					continue;
-				if (IsInvisible(lp->value.cptr) && !member)
-					continue;
-				do_who(sptr, lp->value.cptr, chptr);
-			    }
-		  }
+		    {
+			member = IsMember(sptr, chptr);
+			if (member || !SecretChannel(chptr))
+				for (lp = chptr->members; lp; lp = lp->next)
+				    {
+					if (oper && !IsAnOper(lp->value.cptr))
+						continue;
+					if (IsInvisible(lp->value.cptr) &&
+					    !member)
+						continue;
+					do_who(sptr, lp->value.cptr, chptr);
+				    }
+			penalty += 1;
+		    }
 		else if (chptr && IsAnonymous(chptr) &&
 			 (lp = find_user_link(chptr->members, sptr)))
-				do_who(sptr, lp->value.cptr, chptr);
-	    }
-	else for (acptr = client; acptr; acptr = acptr->next)
-	    {
-		aChannel *ch2ptr = NULL;
-		int	showperson, isinvis;
-
-		if (!IsPerson(acptr))
-			continue;
-		if (oper && !IsAnOper(acptr))
-			continue;
-		showperson = 0;
-		/*
-		 * Show user if they are on the same channel, or not
-		 * invisible and on a non secret channel (if any).
-		 * Do this before brute force match on all relevant fields
-		 * since these are less cpu intensive (I hope :-) and should
-		 * provide better/more shortcuts - avalon
-		 */
-		isinvis = IsInvisible(acptr);
-		for (lp = acptr->user->channel; lp; lp = lp->next)
 		    {
-			chptr = lp->value.chptr;
-			member = IsMember(sptr, chptr);
-			if (isinvis && !member)
-				continue;
-			if (member || (!isinvis && PubChannel(chptr)))
-			    {
-				showperson = 1;
-				if (!IsAnonymous(chptr) || acptr != sptr)
-				    {
-					ch2ptr = chptr;
-					break;
-				    }
-			    }
-			if (HiddenChannel(chptr) && !SecretChannel(chptr) &&
-			    !isinvis)
-				showperson = 1;
+			do_who(sptr, lp->value.cptr, chptr);
+			penalty += 1;
 		    }
-		if (!acptr->user->channel && !isinvis)
-			showperson = 1;
-		/*
-		** This is brute force solution, not efficient...? ;( 
-		** Show entry, if no mask or any of the fields match
-		** the mask. --msa
-		*/
-		if (showperson &&
-		    (!mask ||
-		     match(mask, acptr->name) == 0 ||
-		     match(mask, acptr->user->username) == 0 ||
-		     match(mask, acptr->user->host) == 0 ||
-		     match(mask, acptr->user->server) == 0 ||
-		     match(mask, acptr->info) == 0))
-			do_who(sptr, acptr, ch2ptr);
+	    }
+	else 
+	    {
+		for (acptr = client; acptr; acptr = acptr->next)
+		    {
+			aChannel *ch2ptr = NULL;
+			int	showperson, isinvis;
+			
+			if (!IsPerson(acptr))
+				continue;
+			if (oper && !IsAnOper(acptr))
+				continue;
+			showperson = 0;
+			/*
+			 * Show user if they are on the same channel, or not
+			 * invisible and on a non secret channel (if any).
+			 * Do this before brute force match on all relevant
+			 * fields since these are less cpu intensive (I
+			 * hope :-) and should provide better/more shortcuts
+			 * -avalon
+			 */
+			isinvis = IsInvisible(acptr);
+			for (lp = acptr->user->channel; lp; lp = lp->next)
+			    {
+				chptr = lp->value.chptr;
+				member = IsMember(sptr, chptr);
+				if (isinvis && !member)
+					continue;
+				if (member || (!isinvis && PubChannel(chptr)))
+				    {
+					showperson = 1;
+					if (!IsAnonymous(chptr) ||
+					    acptr != sptr)
+					    {
+						ch2ptr = chptr;
+						break;
+					    }
+				    }
+				if (HiddenChannel(chptr) &&
+				    !SecretChannel(chptr) && !isinvis)
+					showperson = 1;
+			    }
+			if (!acptr->user->channel && !isinvis)
+				showperson = 1;
+			/*
+			 ** This is brute force solution, not efficient...? ;( 
+			 ** Show entry, if no mask or any of the fields match
+			 ** the mask. --msa
+			 */
+			if (showperson &&
+			    (!mask ||
+			     match(mask, acptr->name) == 0 ||
+			     match(mask, acptr->user->username) == 0 ||
+			     match(mask, acptr->user->host) == 0 ||
+			     match(mask, acptr->user->server) == 0 ||
+			     match(mask, acptr->info) == 0))
+				do_who(sptr, acptr, ch2ptr);
+		    }
+		penalty += 3;
 	    }
 	sendto_one(sptr, rpl_str(RPL_ENDOFWHO, parv[0]),
 		   BadPtr(mask) ?  "*" : mask);
-	return 0;
+	return penalty;
 }
 
 /*
@@ -1330,29 +1363,41 @@ char	*parv[];
 	aChannel *chptr;
 	char	*nick, *tmp, *name;
 	char	*p = NULL;
-	int	found, len, mlen;
+	int	found = 0, len, mlen;
 
     	if (parc < 2)
 	    {
 		sendto_one(sptr, err_str(ERR_NONICKNAMEGIVEN, parv[0]));
-		return 0;
+		return 1;
 	    }
 
 	if (parc > 2)
 	    {
 		if (hunt_server(cptr,sptr,":%s WHOIS %s :%s", 1,parc,parv) !=
 		    HUNTED_ISME)
-			return 0;
+			return 2;
 		parv[1] = parv[2];
 	    }
 
+	parv[1] = canonize(parv[1]);
+
 	for (tmp = parv[1]; (nick = strtoken(&p, tmp, ",")); tmp = NULL)
 	    {
 		int	invis, showperson, member, wilds;
 
-		found = 0;
+		found &= 0x0f;	/* high/boolean, low/counter */
 		(void)collapse(nick);
 		wilds = (index(nick, '?') || index(nick, '*'));
+                /*
+		 * We're no longer allowing remote users to generate
+                 * requests with wildcard, nor local users with more
+		 * than one wildcard target per command.
+		 * Max 3 targets per command allowed.
+		 */
+		if ((wilds && (!MyConnect(sptr) || p)) ||
+		    found++ > 3) {
+			break;
+		}
 		for (acptr = client; (acptr = next_client(acptr, nick));
 		     acptr = acptr->next)
 		    {
@@ -1365,20 +1410,13 @@ char	*parv[];
 				break;
 			/*
 			 * 'Rules' established for sending a WHOIS reply:
-			 *
-			 * - only allow a remote client to get replies for
-			 *   local clients if wildcards are being used;
-			 *
 			 * - if wildcards are being used dont send a reply if
 			 *   the querier isnt any common channels and the
 			 *   client in question is invisible and wildcards are
 			 *   in use (allow exact matches only);
-			 *
 			 * - only send replies about common or public channels
 			 *   the target user(s) are on;
 			 */
-			if (!MyConnect(sptr) && !MyConnect(acptr) && wilds)
-				continue;
 			user = acptr->user ? acptr->user : &UnknownUser;
 			name = (!*acptr->name) ? "?" : acptr->name;
 
@@ -1408,7 +1446,7 @@ char	*parv[];
 			sendto_one(sptr, rpl_str(RPL_WHOISUSER, parv[0]),
 				   name, user->username, user->host,
 				   acptr->info);
-			found = 1;
+			found |= 0x10;
 			mlen = strlen(ME) + strlen(parv[0]) + 6 +
 				strlen(name);
 			for (len = 0, *buf = '\0', lp = user->channel; lp;
@@ -1462,7 +1500,7 @@ char	*parv[];
 					   parv[0]), name,
 					   timeofday - user->last);
 		    }
-		if (!found)
+		if (!(found & 0x10))
 		    {
 			if (strlen(nick) > (size_t) NICKLEN)
 				nick[NICKLEN] = '\0';
@@ -1474,7 +1512,7 @@ char	*parv[];
 	    }
 	sendto_one(sptr, rpl_str(RPL_ENDOFWHOIS, parv[0]), parv[1]);
 
-	return 0;
+	return 1;
 }
 
 /*
@@ -1500,7 +1538,7 @@ char	*parv[];
 	if (IsServer(sptr) || IsService(sptr) || sptr->user)
 	    {
 		sendto_one(sptr, err_str(ERR_ALREADYREGISTRED, parv[0]));
-		return 0;
+		return 1;
    	    }
 	if (parc > 2 && (username = (char *)index(parv[1],'@')))
 		*username = '\0'; 
@@ -1512,7 +1550,7 @@ char	*parv[];
 			sendto_flag(SCH_NOTICE,
 				    "bad USER param count for %s from %s",
 				    parv[0], get_client_name(cptr, FALSE));
-		return 0;
+		return 1;
 	    }
 
 	/* Copy parameters into better documenting variables */
@@ -1669,7 +1707,7 @@ char	*parv[];
 	if (parc < 2 || *parv[1] == '\0')
 	    {
 		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS, parv[0]), "KILL");
-		return 0;
+		return 1;
 	    }
 
 	user = parv[1];
@@ -1681,7 +1719,7 @@ char	*parv[];
 		    {
 			sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS, parv[0]),
 				   "KILL");
-			return 0;
+			return 1;
 		    }
 		if (strlen(path) > (size_t) TOPICLEN)
 			path[TOPICLEN] = '\0';
@@ -1699,7 +1737,7 @@ char	*parv[];
 			if (!IsServer(sptr))
 				sendto_one(sptr, err_str(ERR_NOSUCHNICK, 
 							 parv[0]), user);
-			return 0;
+			return 1;
 		    }
 		sendto_one(sptr,":%s NOTICE %s :KILL changed from %s to %s",
 			   ME, parv[0], user, acptr->name);
@@ -1708,7 +1746,7 @@ char	*parv[];
 	if (!MyConnect(acptr) && IsLocOp(cptr))
 	    {
 		sendto_one(sptr, err_str(ERR_NOPRIVILEGES, parv[0]));
-		return 0;
+		return 1;
 	    }
 	if (IsServer(acptr) || IsMe(acptr))
 	    {
@@ -1724,7 +1762,7 @@ char	*parv[];
 	    {
 		sendto_one(sptr, ":%s NOTICE %s :Nick %s isnt on your server",
 			   ME, parv[0], acptr->name);
-		return 0;
+		return 1;
 	    }
 #endif
 	if (!IsServer(cptr))
@@ -1763,7 +1801,7 @@ char	*parv[];
 	if (IsLocOp(sptr) && !MyConnect(acptr))
 	    {
 		sendto_one(sptr, err_str(ERR_NOPRIVILEGES, parv[0]));
-		return 0;
+		return 1;
 	    }
 	sendto_flag(SCH_KILL,
 		    "Received KILL message for %s. From %s Path: %s!%s",
@@ -1875,7 +1913,7 @@ char	*parv[];
 		check_services_butone(SERVICE_WANT_AWAY, NULL, sptr,
 				      ":%s AWAY", parv[0]);
 #endif
-		return 0;
+		return 1;
 	    }
 
 	/* Marking as away */
@@ -1907,7 +1945,7 @@ char	*parv[];
 	sptr->user->away = away;
 	(void)strcpy(away, awy2);
 	sendto_one(sptr, rpl_str(RPL_NOWAWAY, parv[0]));
-	return 0;
+	return 1;
 }
 
 /*
@@ -1927,7 +1965,7 @@ char	*parv[];
 	if (parc < 2 || *parv[1] == '\0')
 	    {
 		sendto_one(sptr, err_str(ERR_NOORIGIN, parv[0]));
-		return 0;
+		return 1;
 	    }
 	origin = parv[1];
 	destination = parv[2]; /* Will get NULL or pointer (parc >= 2!!) */
@@ -1935,7 +1973,7 @@ char	*parv[];
 	acptr = find_client(origin, NULL);
 	if (!acptr)
 		acptr = find_server(origin, NULL);
-	if (acptr && acptr != sptr)
+	if (!acptr || acptr != sptr)
 		origin = cptr->name;
 	if (!BadPtr(destination) && mycmp(destination, ME) != 0)
 	    {
@@ -1946,13 +1984,13 @@ char	*parv[];
 		    {
 			sendto_one(sptr, err_str(ERR_NOSUCHSERVER, parv[0]),
 				   destination);
-			return 0;
+			return 1;
 		    }
 	    }
 	else
-		sendto_one(sptr,":%s PONG %s :%s", ME,
+		sendto_one(sptr, ":%s PONG %s :%s", ME,
 			   (destination) ? destination : ME, origin);
-	return 0;
+	return 1;
     }
 
 /*
@@ -1972,7 +2010,7 @@ char	*parv[];
 	if (parc < 2 || *parv[1] == '\0')
 	    {
 		sendto_one(sptr, err_str(ERR_NOORIGIN, parv[0]));
-		return 0;
+		return 1;
 	    }
 
 	origin = parv[1];
@@ -1990,7 +2028,7 @@ char	*parv[];
 		    {
 			sendto_one(sptr, err_str(ERR_NOSUCHSERVER, parv[0]),
 				   destination);
-			return 0;
+			return 1;
 		    }
 	    }
 #ifdef	DEBUGMODE
@@ -2026,7 +2064,7 @@ char	*parv[];
 	if (!IsServer(cptr) && (BadPtr(name) || BadPtr(password)))
 	    {
 		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS, parv[0]), "OPER");
-		return 0;
+		return 1;
 	    }
 	
 	/* if message arrived from server, trust it, and set to oper */
@@ -2045,13 +2083,13 @@ char	*parv[];
 #ifdef NPATH
                 note_oper(sptr);
 #endif
-		return 0;
+		return 1;
 	    }
 	else if (IsOper(sptr))
 	    {
 		if (MyConnect(sptr))
 			sendto_one(sptr, rpl_str(RPL_YOUREOPER, parv[0]));
-		return 0;
+		return 1;
 	    }
 	if (!(aconf = find_conf_exact(name, sptr->username, sptr->sockhost,
 				      CONF_OPS)) &&
@@ -2059,7 +2097,7 @@ char	*parv[];
 				      inetntoa((char *)&cptr->ip), CONF_OPS)))
 	    {
 		sendto_one(sptr, err_str(ERR_NOOPERHOST, parv[0]));
-		return 0;
+		return 1;
 	    }
 #ifdef CRYPT_OPER_PASSWORD
 	/* use first two chars of the password they send in as salt */
@@ -2099,7 +2137,7 @@ char	*parv[];
 		*--s =  '@';
 		sendto_flag(SCH_NOTICE, "%s (%s@%s) is now operator (%c)",
 			    parv[0], sptr->user->username, sptr->user->host,
-			   IsOper(sptr) ? 'O' : 'o');
+			   IsOper(sptr) ? 'o' : 'O');
 		if (IsOper(sptr))
 			send_umode_out(cptr, sptr, old);
  		sendto_one(sptr, rpl_str(RPL_YOUREOPER, parv[0]));
@@ -2149,7 +2187,7 @@ char	*parv[];
 #ifdef NPATH
                 note_oper(sptr);
 #endif
-		if (IsOper(sptr))
+		if (IsAnOper(sptr))
 			istat.is_oper++;
 	    }
 	else
@@ -2157,7 +2195,7 @@ char	*parv[];
 		(void)detach_conf(sptr, aconf);
 		sendto_one(sptr,err_str(ERR_PASSWDMISMATCH, parv[0]));
 	    }
-	return 0;
+	return 1;
     }
 
 /***************************************************************************
@@ -2180,7 +2218,7 @@ char	*parv[];
 	if (BadPtr(password))
 	    {
 		sendto_one(cptr, err_str(ERR_NEEDMOREPARAMS, parv[0]), "PASS");
-		return 0;
+		return 1;
 	    }
 	if (parc > 2 && parv[2])
 		strncpyzt(cptr->info, parv[2], NICKLEN);
@@ -2208,7 +2246,7 @@ char	*parv[];
 	    {
 		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS, parv[0]),
 			   "USERHOST");
-		return 0;
+		return 1;
 	    }
 
 	(void)strcpy(buf, rpl_str(RPL_USERHOST, parv[0]));
@@ -2244,7 +2282,7 @@ char	*parv[];
 		    }
 	    }
 	sendto_one(sptr, "%s", buf);
-	return 0;
+	return 1;
 }
 
 /*
@@ -2270,7 +2308,7 @@ char	*parv[];
 	if (parc < 2)
 	    {
 		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS, parv[0]), "ISON");
-		return 0;
+		return 1;
 	    }
 
 	(void)strcpy(buf, rpl_str(RPL_ISON, *parv));
@@ -2284,7 +2322,7 @@ char	*parv[];
 			(void) strcpy(buf + len++, " ");
 		    }
 	sendto_one(sptr, "%s", buf);
-	return 0;
+	return 1;
 }
 
 #if !defined(NPATH)
@@ -2300,7 +2338,7 @@ char	*parv[];
 	char	*c, nbuf[50];
 
 	if (parc < 2)
-		return 0;
+		return 1;
 
 	c = parv[1];
 
@@ -2331,7 +2369,7 @@ char	*parv[];
 					   parv[0], parv[1]);
 				break;
 			    }
-	return 0;
+	return 1;
 }
 #endif
 
@@ -2356,15 +2394,15 @@ char	*parv[];
 	Reg	int	flag;
 	Reg	int	*s;
 	Reg	char	**p, *m;
-	aClient	*acptr;
-	int	what, setflags;
+	aClient	*acptr = NULL;
+	int	what, setflags, penalty = 0;
 
 	what = MODE_ADD;
 
 	if (parc < 2)
 	    {
 		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS, parv[0]), "MODE");
-		return 0;
+		return 1;
 	    }
 
 	if (cptr && !(acptr = find_person(parv[1], NULL)))
@@ -2372,7 +2410,7 @@ char	*parv[];
 		if (MyConnect(sptr))
 			sendto_one(sptr, err_str(ERR_NOSUCHCHANNEL, parv[0]),
 				   parv[1]);
-		return 0;
+		return 1;
 	    }
 	if (cptr == NULL)
 		/* internal call which has to be handled in a special way */
@@ -2388,7 +2426,7 @@ char	*parv[];
 				  get_client_name(cptr, FALSE), sptr->name);
 		else
 			sendto_one(sptr, err_str(ERR_USERSDONTMATCH, parv[0]));
-			return 0;
+			return 1;
 	    }
  
 	if (parc < 3)
@@ -2401,7 +2439,7 @@ char	*parv[];
 				*m++ = (char)(*(s+1));
 		*m = '\0';
 		sendto_one(sptr, rpl_str(RPL_UMODEIS, parv[0]), buf);
-		return 0;
+		return 1;
 	    }
 
 	/* find flags already set for user */
@@ -2459,12 +2497,11 @@ char	*parv[];
 		    !IsServer(cptr))
 			sptr->user->flags &= ~FLAGS_LOCOP;
 		if ((setflags & FLAGS_RESTRICTED) &&
-		    !(sptr->user->flags & FLAGS_RESTRICTED) &&
-		    (what == MODE_DEL))
+		    !(sptr->user->flags & FLAGS_RESTRICTED))
 		    {
 			sendto_one(sptr, err_str(ERR_RESTRICTED, parv[0]));
 			SetRestricted(sptr);
-			return 0;
+			/* Can't return; here since it could mess counters */
 		    }
 		if ((setflags & (FLAGS_OPER|FLAGS_LOCOP)) && !IsAnOper(sptr) &&
 		    MyConnect(sptr))
@@ -2478,11 +2515,13 @@ char	*parv[];
 		    {
 			istat.is_user[1]--;
 			istat.is_user[0]++;
+			penalty += 1;
 		    }
 		if (IsInvisible(sptr) && !(setflags & FLAGS_INVISIBLE))
 		    {
 			istat.is_user[1]++;
 			istat.is_user[0]--;
+			penalty += 1;
 		    }
 		send_umode_out(cptr, sptr, setflags);
 	    }
@@ -2510,7 +2549,7 @@ char	*parv[];
 	if (IsOper(sptr) && !(setflags & FLAGS_OPER))
 		note_oper(sptr);
 #endif
-	return 0;
+	return penalty;
 }
 	
 /*
